/*
 * include/asm-microblaze/entry.h
 *
 * Definitions used by low-level trap handlers
 *
 * Copyright (C) 2008 Michal Simek
 * Copyright (C) 2007 - 2008 PetaLogix
 * Copyright (C) 2007 John Williams <john.williams@petalogix.com>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License. See the file COPYING in the main directory of this
 * archive for more details.
 *
 */

#ifndef _ASM_MICROBLAZE_ENTRY_H
#define _ASM_MICROBLAZE_ENTRY_H

#include <asm/percpu.h>
#include <asm/ptrace.h>

/* These are per-cpu variables required in entry.S, among other
 places */

#define PER_CPU(var) per_cpu__##var

#ifndef __ASSEMBLY__
DECLARE_PER_CPU(unsigned int, KSP); /* Saved kernel stack pointer */
DECLARE_PER_CPU(unsigned int, KM); /* Kernel/user mode */
DECLARE_PER_CPU(unsigned int, ENTRY_SP); /* Saved SP on kernel entry */
DECLARE_PER_CPU(unsigned int, R11_SAVE); /* Temp variable for entry */
DECLARE_PER_CPU(unsigned int, CURRENT_SAVE); /* Saved current pointer */
DECLARE_PER_CPU(unsigned int, SYSCALL_SAVE); /* Saved syscall number */
#ifdef CONFIG_MMU
DECLARE_PER_CPU(unsigned int, SCRATCH); /* Helping for storage some data */
DECLARE_PER_CPU(unsigned int, NMI_ENTRY_SP); /* Helping for storage some data */
#endif /* CONFIG_MMU */
#endif /* __ASSEMBLY__ */

#ifdef CONFIG_MMU

#ifdef __ASSEMBLY__
#define R0_RAM_ADDR (r0_ram)
#else
extern char r0_ram;
#define R0_RAM_ADDR ((unsigned int)&r0_ram)
#endif

/* If true, system calls save and restore all registers (except result
registers, of course).  If false, then `call clobbered' registers
will not be preserved, on the theory that system calls are basically
function calls anyway, and the caller should be able to deal with it.
This is a security risk, of course, as `internal' values may leak out
after a system call, but that certainly doesn't matter very much for
a processor with no MMU protection!  For a protected-mode kernel, it
would be faster to just zero those registers before returning.  */
/* I can not rely on the glibc implementation. If you turn it off make sure that r11/r12 is saved in user-space. --KAA */
#define TRAPS_PRESERVE_CALL_CLOBBERED_REGS	1

/* If TRAPS_PRESERVE_CALL_CLOBBERED_REGS is false, then zero `call
clobbered' registers before returning from a system call.  */
#define TRAPS_ZERO_CALL_CLOBBERED_REGS		0

/* These are special variables using by the kernel trap/interrupt code
to save registers in, at a time when there are no spare registers we
can use to do so, and we can't depend on the value of the stack
pointer.  This means that they must be within a signed 16-bit
displacement of 0x00000000.  */

#define KERNEL_VAR_SPACE_ADDR	R0_RAM_ADDR

/* 
* KERNEL_VAR is from the microblaze port.  To maintain compatability
* we keep it, but the mb-gas doesn't do the reg[offset] syntax. 
* instead, we should get away with something like addi reg, r0, offset.
* Here's hoping.
*/
#ifdef __ASSEMBLY__
/* #define KERNEL_VAR(addr)	addr[r0] */
#define KERNEL_VAR(addr)	(addr)
#else
#define KERNEL_VAR(addr)	(*(volatile unsigned long *)(addr))
#endif

/* Provide storage for fast execution of immediate expection handlers (not to calculate the real stack pointer)*/
/* these addrs have to be in order! unaligned_ex relies on it!*/
#define REG_SAVE_ADDR(regnum)		(KERNEL_VAR_SPACE_ADDR + 28 + (regnum*4))
#define REG_SAVE(regnum)			KERNEL_VAR (REG_SAVE_ADDR(regnum))

/* A `state save frame' is a struct pt_regs preceded by some extra space
suitable for a function call stack frame. */

/* Amount of room on the stack reserved for arguments and to satisfy the
C calling conventions, in addition to the space used by the struct
pt_regs that actually holds saved values. */
#define STATE_SAVE_ARG_SPACE	(6*4) /* Up to six arguments */

#ifdef __ASSEMBLY__
/* The size of a state save frame.  */
#define STATE_SAVE_SIZE		(PT_SIZE + STATE_SAVE_ARG_SPACE)
#else /* !__ASSEMBLY__ */
/* The size of a state save frame.  */
#define STATE_SAVE_SIZE	       (sizeof (struct pt_regs) + STATE_SAVE_ARG_SPACE)
#endif /* __ASSEMBLY__ */

/* Offset of the struct pt_regs in a state save frame.  */
#define STATE_SAVE_PT_OFFSET	STATE_SAVE_ARG_SPACE

/* to compile with old assembler */
#define rPID 0x1000
#define rZPR 0x1001
#define rTLBX 0x1002
#define rTLBLO 0x1003
#define rTLBHI 0x1004
#define rTLBSX 0x1005

#define MTS(spr, gpr) .long (0x9400C000 | gpr<<16 | spr)
#define MFS(gpr, spr) .long (0x94008000 | gpr<<21 | spr)

#endif /* CONFIG_MMU */

#endif /* _ASM_MICROBLAZE_ENTRY_H */
