/*
 * include/asm-microblaze/processor.h
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License. See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2008 Michal Simek
 * Copyright (C) 2008 PetaLogix
 * Copyright (C) 2006 Atmark Techno, Inc.
 */

#ifndef _ASM_MICROBLAZE_PROCESSOR_H
#define _ASM_MICROBLAZE_PROCESSOR_H

#include <linux/autoconf.h>
#include <asm/ptrace.h>
#include <asm/setup.h>
#include <asm/registers.h>
#include <asm/segment.h>

#ifdef CONFIG_MMU
#include <asm/entry.h>
#include <asm/current.h>
#endif

#ifndef CONFIG_MMU
/*
 * User space process size: memory size
 *
 * TASK_SIZE on MMU cpu is usually 1GB. However, on no-MMU arch, both
 * user processes and the kernel is on the same memory region. They
 * both share the memory space and that is limited by the amount of
 * physical memory. thus, we set TASK_SIZE == amount of total memory.
 */

#define TASK_SIZE	(0x81000000 - 0x80000000)

/*
 * Default implementation of macro that returns current
 * instruction pointer ("program counter").
 */
#define current_text_addr() ({ __label__ _l; _l: &&_l; })

/*
 * This decides where the kernel will search for a free chunk of vm
 * space during mmap's. We won't be using it
 */
#define TASK_UNMAPPED_BASE	0

struct task_struct;

/* thread_struct is gone. use thread_info instead. */
struct thread_struct { };
#define INIT_THREAD	{ }

/* Do necessary setup to start up a newly executed thread. */
static inline void start_thread(struct pt_regs *regs,
				unsigned long pc,
				unsigned long usp)
{
	regs->pc = pc;
	regs->r1 = usp;
	regs->kernel_mode = 0;
}

/* Free all resources held by a thread. */
static inline void release_thread(struct task_struct *dead_task)
{
}

/* Free all resources held by a thread. */
static inline void exit_thread(void)
{
}

extern unsigned long thread_saved_pc(struct task_struct *t);

extern unsigned long get_wchan(struct task_struct *p);

/* FIXME */
#define cpu_relax()		do {} while (0)
#define prepare_to_copy(tsk)	do {} while (0)

/*
 * create a kernel thread without removing it from tasklists
 */
extern int kernel_thread(int (*fn)(void *), void *arg, unsigned long flags);

/* #define task_thread_info(task) (task)->thread_info */
/* #define task_stack_page(task) ((void*)((task)->thread_info)) */

#define task_pt_regs(tsk) \
		(((struct pt_regs *)(THREAD_SIZE + task_stack_page(tsk))) - 1)

#define KSTK_EIP(tsk)	(0)
#define KSTK_ESP(tsk)	(0)

#else /* CONFIG_MMU */

/*
 * This is used to define STACK_TOP, and with MMU it must be below
 * kernel base to select the correct PGD when handling MMU exceptions.
 */
#define TASK_SIZE	(CONFIG_KERNEL_START)


/* This decides where the kernel will search for a free chunk of vm
 * space during mmap's.
 */
#define TASK_UNMAPPED_BASE	(TASK_SIZE / 8 * 3)

#define THREAD_KSP	0

#ifndef __ASSEMBLY__

/*
 * Default implementation of macro that returns current
 * instruction pointer ("program counter").
 */
#define current_text_addr()	({ __label__ _l; _l: &&_l;})

/*
 * Bus types
 */
#define EISA_bus 0
#define EISA_bus__is_a_macro /* for versions in ksyms.c */
#define MCA_bus 0
#define MCA_bus__is_a_macro /* for versions in ksyms.c */

/* If you change this, you must change the associated assembly-languages
 * constants defined below, THREAD_*.
 */
struct thread_struct {
	/* kernel stack pointer (must be first field in structure) */
	unsigned long  ksp;
	mm_segment_t	fs;		/* for get_fs() validation */
	void		*pgdir;		/* root of page-table tree */
	struct pt_regs	*regs;		/* Pointer to saved register state */
};

#define INIT_THREAD { \
	.ksp   = sizeof init_stack + (unsigned long)init_stack, \
	.fs    = KERNEL_DS, \
	.pgdir = swapper_pg_dir, \
}

/* Do necessary setup to start up a newly executed thread.  */
void start_thread (struct pt_regs *regs,
		unsigned long pc, unsigned long usp);

/* Free all resources held by a thread. */
extern inline void release_thread (struct task_struct *dead_task)
{
}

extern int kernel_thread(int (*fn)(void *), void * arg, unsigned long flags);

/* Free current thread data structures etc.  */
static inline void exit_thread (void)
{
}

/* Return saved (kernel) PC of a blocked thread.  */
#define thread_saved_pc(tsk)	\
	((tsk)->thread.regs? (tsk)->thread.regs->r15: 0)

unsigned long get_wchan (struct task_struct *p);

/* Return some info about the user process TASK.  */
#define task_tos(task)	((unsigned long)(task) + KERNEL_STACK_SIZE)
#define task_regs(task) ((struct pt_regs *)task_tos (task) - 1)

#define task_pt_regs(tsk) (((struct pt_regs *)(THREAD_SIZE + task_stack_page(tsk))) - 1)
#define task_pt_regs_plus_args(tsk) (((void *)task_pt_regs(tsk)) - STATE_SAVE_ARG_SPACE)

#define task_thread_info(task) (task)->thread_info
#define task_stack_page(task) ((void*)((task)->thread_info))
#define task_sp(task)	(task_regs (task)->sp)
#define task_pc(task)	(task_regs (task)->pc)
/* Grotty old names for some.  */
#define KSTK_EIP(task)	task_pc (task)
#define KSTK_ESP(task)	task_sp (task)

#define cpu_relax() barrier()

/* FIXME */
#define prepare_to_copy(tsk)	do {} while(0)
#define deactivate_mm(tsk,mm)	do { } while (0)

#endif /* __ASSEMBLY__ */

#endif /* CONFIG_MMU */

#endif /* _ASM_MICROBLAZE_PROCESSOR_H */
