/*
 * include/asm-microblaze/ptrace.h
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License. See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2006 Atmark Techno, Inc.
 */

#ifndef _ASM_MICROBLAZE_PTRACE_H
#define _ASM_MICROBLAZE_PTRACE_H

#include <asm/types.h>
#include <linux/autoconf.h>

#ifdef CONFIG_MMU

/* Microblaze general purpose registers with special meanings.  */
#define GPR_ZERO	0	/* constant zero */
#define GPR_ASM		18	/* reserved for assembler */
#define GPR_SP		1	/* stack pointer */
#define GPR_GP		2	/* global data pointer */
#define GPR_EP		30	/* `element pointer' */
#define GPR_LP		15	/* link pointer (current return address) */

/* These aren't official names, but they make some code more descriptive.  */
#define GPR_ARG0	5
#define GPR_ARG1	6
#define GPR_ARG2	7
#define GPR_ARG3	8
#define GPR_ARG4	9
#define GPR_ARG5	10
#define GPR_RVAL0	3	
#define GPR_RVAL1	4
#define GPR_RVAL	GPR_RVAL0

#define NUM_GPRS	32

/* `system' registers.  */
/* Note these are old v850 values, microblaze has many fewer */
#define SR_EIPC		0
#define SR_EIPSW	1
#define SR_FEPC		2
#define SR_FEPSW	3
#define SR_ECR		4
#define SR_PSW		5
#define SR_CTPC		16
#define SR_CTPSW	17
#define SR_DBPC		18
#define SR_DBPSW	19
#define SR_CTBP		20
#define SR_DIR		21
#define SR_ASID		23

#define instruction_pointer(regs)	((regs)->pc)

/* When a struct pt_regs is used to save user state for a system call in
the kernel, the system call is stored in the space for R0 (since it's
never used otherwise, R0 being a constant 0).  Non-system-calls
simply store 0 there.  */
#define PT_REGS_SYSCALL(regs)		(regs)->r0
#define PT_REGS_SET_SYSCALL(regs, val)	((regs)->r0 = (val))
/* If you change this structure don't forget to change offsets below*/

/* The number of bytes used to store each register.  */
#define _PT_REG_SIZE	4

/* Offset of a general purpose register in a stuct pt_regs.  */
#define PT_GPR(num)	((num) * _PT_REG_SIZE)

/* Offsets of various special registers & fields in a struct pt_regs.  */
/*
#define NUM_SPECIAL	7
#define PT_PC		((NUM_GPRS + 0) * _PT_REG_SIZE)
#define PT_MSR		((NUM_GPRS + 1) * _PT_REG_SIZE)
#define PT_EAR		((NUM_GPRS + 2) * _PT_REG_SIZE)
#define PT_ESR		((NUM_GPRS + 3) * _PT_REG_SIZE)
#define PT_FSR		((NUM_GPRS + 4) * _PT_REG_SIZE)
#define PT_KERNEL_MODE	((NUM_GPRS + 5) * _PT_REG_SIZE)
#define PT_SINGLESTEP	((NUM_GPRS + 6) * _PT_REG_SIZE)
*/

#define PT_SYSCALL	PT_GPR(0)

/* Size of struct pt_regs, including alignment.  */
//#define PT_SIZE		((NUM_GPRS + NUM_SPECIAL) * _PT_REG_SIZE)

/* These are `magic' values for PTRACE_PEEKUSR that return info about where
a process is located in memory.  */
/*#define PT_TEXT_ADDR    (PT_SIZE + 1)
#define PT_TEXT_LEN     (PT_SIZE + 2)
#define PT_DATA_ADDR    (PT_SIZE + 3)
#define PT_DATA_LEN     (PT_SIZE + 4)*/

#ifdef __KERNEL__
	/* Arbitrarily choose the same ptrace numbers as used by the Sparc code. */
	#define PTRACE_GETREGS            12
	#define PTRACE_SETREGS            13
	#define PTRACE_GETFPREGS          14
	#define PTRACE_SETFPREGS          15
	#define PTRACE_GETFPXREGS         18
	#define PTRACE_SETFPXREGS         19
	
	#define PTRACE_OLDSETOPTIONS         21
	
	#define PTRACE_GET_THREAD_AREA    25
	#define PTRACE_SET_THREAD_AREA    26
#endif /* __KERNEL__ */

#endif /* CONFIG_MMU */

#ifndef __ASSEMBLY__

typedef unsigned long microblaze_reg_t;

struct pt_regs {
	microblaze_reg_t r0;
#ifndef CONFIG_MMU
	microblaze_reg_t r1;
#else
	microblaze_reg_t sp;
#endif
	microblaze_reg_t r2;
	microblaze_reg_t r3;
	microblaze_reg_t r4;
	microblaze_reg_t r5;
	microblaze_reg_t r6;
	microblaze_reg_t r7;
	microblaze_reg_t r8;
	microblaze_reg_t r9;
	microblaze_reg_t r10;
	microblaze_reg_t r11;
	microblaze_reg_t r12;
	microblaze_reg_t r13;
	microblaze_reg_t r14;
	microblaze_reg_t r15;
	microblaze_reg_t r16;
	microblaze_reg_t r17;
	microblaze_reg_t r18;
	microblaze_reg_t r19;
	microblaze_reg_t r20;
	microblaze_reg_t r21;
	microblaze_reg_t r22;
	microblaze_reg_t r23;
	microblaze_reg_t r24;
	microblaze_reg_t r25;
	microblaze_reg_t r26;
	microblaze_reg_t r27;
	microblaze_reg_t r28;
	microblaze_reg_t r29;
	microblaze_reg_t r30;
	microblaze_reg_t r31;
	microblaze_reg_t pc;
	microblaze_reg_t msr;
	microblaze_reg_t ear;
	microblaze_reg_t esr;
	microblaze_reg_t fsr;
	int kernel_mode;
};

#define kernel_mode(regs)		((regs)->kernel_mode)
#define user_mode(regs)			(!kernel_mode(regs))

#define instruction_pointer(regs)	((regs)->pc)
#define profile_pc(regs)		instruction_pointer(regs)

extern void show_regs(struct pt_regs *);

#endif /* __ASSEMBLY__ */

#endif /* _ASM_MICROBLAZE_PTRACE_H */
